@@ -6,7 +6,7 @@ Mojo::IRC - IRC Client for the Mojo IOLoop
 
 =head1 VERSION
 
-0.0301
+0.05
 
 =head1 SYNOPSIS
 
@@ -36,14 +36,27 @@ Mojo::IRC - IRC Client for the Mojo IOLoop
 
 =head1 DESCRIPTION
 
+L<Mojo::IRC> is a non-blocking IRC client using L<Mojo::IOLoop> from the
+wonderful L<Mojolicious> framework.
+
+If features IPv6 and TLS, with additional optional modules:
+L<IO::Socket::IP> and L<IO::Socket::SSL>.
+
+By default this module will only emit standard IRC events, but by
+settings L</parser> to a custom object it will also emit CTCP events.
+Example:
+
+  my $irc = Mojo::IRC->new;
+  $irc->parser(Parse::IRC->new(ctcp => 1);
+  $irc->on(ctcp_action => sub {
+    # ...
+  });
+
+It will also set up some default events: L</ctcp_ping>, L</ctcp_time>,
+and L</ctcp_version>.
+
 This class inherit from L<Mojo::EventEmitter>.
 
-TODO:
-
-  * Authentication with password
-  * SSL
-  * use IRC::Utils qw/ decode_irc /;
-
 =head1 EVENTS
 
 =head2 close
@@ -77,7 +90,7 @@ This event is used by IRC errors
 
   $self->$callback({
     params => ['newnick'],
-    raw_line => ':oldnick!~someuser@hostname.com MODE somenick :+i',
+    raw_line => ':oldnick!~someuser@hostname.com NICK :newnick',
     command => 'NICK',
     prefix => 'somenick!~someuser@hostname.com'
   });
@@ -254,9 +267,14 @@ use constant DEBUG => $ENV{MOJO_IRC_DEBUG} ? 1 : 0;
 use constant DEFAULT_CERT => $ENV{MOJO_IRC_CERT_FILE} || catfile dirname(__FILE__), 'mojo-irc-client.crt';
 use constant DEFAULT_KEY => $ENV{MOJO_IRC_KEY_FILE} || catfile dirname(__FILE__), 'mojo-irc-client.key';
 
-our $VERSION = '0.0301';
+our $VERSION = '0.05';
 
-my @DEFAULT_EVENTS = qw/irc_ping irc_nick irc_notice irc_rpl_welcome irc_err_nicknameinuse/;
+my %CTCP_QUOTE = ( "\012" => 'n', "\015" => 'r', "\0" => '0', "\cP" => "\cP" );
+
+my @DEFAULT_EVENTS = qw(
+  irc_ping irc_nick irc_notice irc_rpl_welcome irc_err_nicknameinuse
+  ctcp_ping ctcp_time ctcp_version
+);
 
 =head1 ATTRIBUTES
 
@@ -322,6 +340,18 @@ The name of this IRC client. Defaults to "Mojo IRC".
 
 has name => 'Mojo IRC';
 
+=head2 parser
+
+  $self = $self->parser($obj);
+  $self = $self->parser(Parse::IRC->new(ctcp => 1));
+  $obj = $self->parser;
+
+Holds a L<Parse::IRC> object by default.
+
+=cut
+
+has parser => sub { Parse::IRC->new; };
+
 =head2 pass
 
 Password for authentication
@@ -354,18 +384,20 @@ has tls => undef;
 
 =head2 change_nick
 
-Change IRC nick name. This command will change the L</nick> attribute if not
-connected to the server.
+This will be deprecated. Use the code below instead:
+
+  $self->write(NICK => $new_nick);
 
 =cut
 
 sub change_nick {
   my ($self, $nick) = @_;
-  my $old = $self->nick // '';
 
-  return $self unless defined $nick;
-  return $self if $old and $old eq $nick;
-  $self->write(NICK => $nick, sub { $_[1] and $self->nick($nick) });
+  warn "change_nick() is deprecated";
+
+  return $self unless length $nick;
+  return $self if $self->nick eq $nick;
+  $self->write(NICK => $nick);
   $self;
 }
 
@@ -441,32 +473,34 @@ sub connect {
 
           while ($buffer =~ s/^([^\r\n]+)\r\n//m) {
             warn "[$self->{debug_key}] >>> $1\n" if DEBUG;
-            $message = Parse::IRC::parse_irc($1);
+            $message = $self->parser->parse($1);
             $method = $message->{command} || '';
 
             if ($method =~ /^\d+$/) {
               $method = IRC::Utils::numeric_to_name($method);
             }
+            if ($method !~ /^CTCP_/) {
+              $method = "irc_$method";
+            }
 
-            $self->emit_safe(lc('irc_' . $method) => $message);
-            $self->emit_safe('irc_error' => $message) if $method =~ m/^err_/i;
+            $self->emit_safe(lc $method, $message);
+            $self->emit_safe('irc_error', $message) if $method =~ m/^err_/i;
           }
         }
       );
 
       $self->{stream} = $stream;
       $self->ioloop->delay(
+        sub {
+          return $self->write(PASS => $self->pass, shift->begin) if length $self->pass;
+          return shift->begin->();
+        },
         sub {
           $self->write(NICK => $self->nick, shift->begin);
         },
         sub {
           $self->write(USER => $self->user, 8, '*', ':' . $self->name, shift->begin);
         },
-        sub {
-          my $delay = shift;
-          return $delay->begin->() unless $self->pass;
-          return $self->write(PASS => $self->pass, $delay->begin);
-        },
         sub {
           $self->$cb('');
         }
@@ -475,6 +509,28 @@ sub connect {
   );
 }
 
+=head2 ctcp
+
+  $str = $self->ctcp(@str);
+
+This message will quote CTCP messages. Example:
+
+  $self->write(PRIVMSG => nickname => $self->ctcp(TIME => time));
+
+The code above will write this message to IRC server:
+
+  PRIVMSG nickname :\001TIME 1393006707\001
+
+=cut
+
+sub ctcp {
+  my $self = shift;
+  local $_ = join ' ', @_;
+  s/([\012\015\0\cP])/\cP$CTCP_QUOTE{$1}/g;
+  s/\001/\\a/g;
+  ":\001${_}\001";
+}
+
 =head2 disconnect
 
   $self->disconnect(\&callback);
@@ -557,6 +613,58 @@ sub write {
 
 =head1 DEFAULT EVENT HANDLERS
 
+=head2 ctcp_ping
+
+Will respond to the sender with the difference in time.
+
+  Ping reply from $sender: 0.53 second(s)
+
+=cut
+
+sub ctcp_ping {
+  my ($self, $message) = @_;
+  my $t0 = $message->{params}[1] || '';
+
+  return $self unless $t0 =~ /^\d+$/;
+  return $self->write(
+    'NOTICE',
+    $message->{params}[0],
+    $self->ctcp(sprintf "Ping reply from %s: %s second(s)", $self->nick, time - $t0),
+  );
+}
+
+=head2 ctcp_time
+
+Will respond to the sender with the current localtime. Example:
+
+  TIME Fri Feb 21 18:56:50 2014
+
+NOTE! The localtime format may change.
+
+=cut
+
+sub ctcp_time {
+  my ($self, $message) = @_;
+
+  $self->write(NOTICE => $message->{params}[0], $self->ctcp(TIME => scalar localtime));
+}
+
+=head2 ctcp_version
+
+Will respond to the sender with:
+
+  VERSION Mojo-IRC $VERSION
+
+NOTE! Additional information may be added later on.
+
+=cut
+
+sub ctcp_version {
+  my ($self, $message) = @_;
+
+  $self->write(NOTICE => $message->{params}[0], $self->ctcp(VERSION => 'Mojo-IRC', $VERSION));
+}
+
 =head2 irc_nick
 
 Used to update the L</nick> attribute when the nick has changed.
@@ -621,16 +729,15 @@ sub irc_rpl_welcome {
 
 =head2 irc_err_nicknameinuse
 
-Tries to register with the same nick as L</nick>, only with an extra underscore
-added. The new nick will be stored in L</nick>.
+This handler will add "_" to the failed nick before trying to register again.
 
 =cut
 
 sub irc_err_nicknameinuse {
   my ($self, $message) = @_;
+  my $nick = $message->{params}[1];
 
-  $self->nick($self->nick . '_');
-  $self->write(NICK => $self->nick);
+  $self->write(NICK => $nick .'_');
 }
 
 sub DESTROY {
@@ -657,5 +764,3 @@ Jan Henning Thorsen - C<jhthorsen@cpan.org>
 =cut
 
 1;
-
-1;
